#include <iostream>
#include <vector>
#include "histogram.h"
#include "text.h"
#include "svg.h"
#include <curl/curl.h>
#include <sstream>
#include <string>

using namespace std;

struct Input
{
    vector<double> numbers;
    size_t bin_count{};
};

size_t write_data(void* items, size_t item_size, size_t item_count, void* ctx) {
     size_t data_size = item_size * item_count;
     stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
     buffer->write(reinterpret_cast<const char*>(items), data_size);
    return data_size;
}

Input
input_data(istream& in, bool prompt)
{
    Input cin;
    string numbCntPrmt, numbsPrmt, binCntPrmt;
    if (prompt == true) {
        numbCntPrmt = "Enter number count: ";
        numbsPrmt = "Enter numbers: ";
        binCntPrmt = "Enter bin count: ";
    }
    size_t number_count;
    cerr << numbCntPrmt;
    in >> number_count;
    cin.numbers.resize(number_count);

    vector<double> numbers(number_count);

    cerr << numbsPrmt;
    for (size_t i = 0; i < number_count; i++)
    {
        in >> cin.numbers[i];
    }

    cerr << binCntPrmt;
    in >> cin.bin_count;
    return cin;
}

Input
download(const string& address) {
    stringstream buffer;

    CURL* curl = curl_easy_init();
    if(curl){
        CURLcode res;
        curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);

        res = curl_easy_perform(curl);

        if(!res) {
            curl_off_t ul;
            res = curl_easy_getinfo(curl, CURLINFO_SIZE_DOWNLOAD_T, &ul);
            if(!res) {
                cerr<<"Uploaded bytes" << ul << endl;
            }
        }

        if (res != 0){
            cerr << curl_easy_strerror(res);
            exit(1);
        }
        curl_easy_cleanup(curl);
    }
    return input_data(buffer, false);
}

int main(int argc, char* argv[])
{
    Input input;
    if (argc > 1) {
        input = download(argv[1]);
    } else {
        input = input_data(cin, true);
    }

    size_t max_count;
    auto in = input_data(cin, true);
    auto bins = make_histogram(in.numbers, in.bin_count);
    show_histogram_svg(bins);
}
